;*** cmd_exe.inc ***;
;------------------------------------------------
; Author: Torsten Knorr
; E-mail: create-soft@freenet.de
; copyright(C) 2016-2017 by Torsten Knorr
;------------------------------------------------

 .ifndef __RFM12BP_CMD_EXE__
 .equ    __RFM12BP_CMD_EXE__ = 1

 .include "RFM12BP.inc"

;------------------------------------------------

 .equ RFM12BP_REMOTE_CMD  = 7
 .equ RFM12BP_LOCAL_CMD   = 6
 .equ RFM12BP_CMD_MASK    = 0x3F

 .equ CONFIRM_TIMEOUT_1S = 0x26
 .equ CONFIRM_TIMEOUT_5S = 0xBF
 .equ CONFIRM_TIMEOUT_6S = 0xE5

;------------------------------------------------

 .macro M_RFM12BP_CHECK_CMD

    lds  R_TEMP_L,      s_cmd
    cpi  R_TEMP_L,      0x00
    breq NO_COMMAND

 .ifdef __RFM12BP_REQUEST__

    sbrs R_TEMP_L,      RFM12BP_LOCAL_CMD
    rjmp NO_COMMAND

 .endif

 .ifdef __RFM12BP_RESPONSE__

    sbrs R_TEMP_L,      RFM12BP_REMOTE_CMD
    rjmp NO_COMMAND

 .endif

    andi R_TEMP_L,      RFM12BP_CMD_MASK

    cpi  R_TEMP_L,      TABLE_END
    brsh NO_COMMAND

    ldi  R_ZP_L,        low(RFM12BP_CMD_EXE_TABLE)
    ldi  R_ZP_H,        high(RFM12BP_CMD_EXE_TABLE)
    ldi  R_TEMP_H,      0x00

    add  R_ZP_L,        R_TEMP_L
    adc  R_ZP_H,        R_TEMP_H
    icall

 NO_COMMAND:

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_CMD_EXE_TABLE

    rjmp RFM12BP_NOP        ; 0x00
    rjmp RFM12BP_CONF_OUT_1 ; 0x01
    rjmp RFM12BP_CONF_OUT_2 ; 0x02
    rjmp RFM12BP_CONF_DEV_1 ; 0x03
    rjmp RFM12BP_CONF_DEV_2 ; 0x04
    rjmp RFM12BP_CONF_BBW   ; 0x05
    rjmp RFM12BP_CONF_LNAG  ; 0x06
    rjmp RFM12BP_CONF_DRSSI ; 0x07
    rjmp RFM12BP_CONF_DQD   ; 0x08
    rjmp RFM12BP_CONFIRM    ; 0x09
    rjmp RFM12BP_REMOTE_CMD ; 0x0A
    rjmp RFM12BP_LOAD_DEF   ; 0x0B
    rjmp RFM12BP_MODUL_RES  ; 0x0C
    rjmp RFM12BP_COMPL_RES  ; 0x0D
    rjmp TEST_LEDS          ; 0x0E
    rjmp TEST_LED_DEBUG_1   ; 0x0F
    rjmp TEST_LED_DEBUG_2   ; 0x10
    rjmp RFM12BP_END        ; 0x11

 .equ TABLE_END = 0x11

;------------------------------------------------

 RFM12BP_NOP:

    ret

;------------------------------------------------

 RFM12BP_CONF_OUT_1:

    lds  R_TEMP_L,      s_para_1
    andi R_TEMP_L,      0x07

    in   R_TEMP_H,      TX_CONFIG_1
    andi R_TEMP_H,      ~0x07

    or   R_TEMP_L,      R_TEMP_H
    out  TX_CONFIG_1,   R_TEMP_L

    ldi  R_TEMP_H,      e_tx_config_1
    rcall EEPROM_WRITE_BYTE

 ret

;------------------------------------------------

 RFM12BP_CONF_OUT_2:

    lds  R_TEMP_L,      s_para_1
    andi R_TEMP_L,      0x07

    in   R_TEMP_H,      TX_CONFIG_2
    andi R_TEMP_H,      ~0x07

    or   R_TEMP_L,      R_TEMP_H
    out  TX_CONFIG_2,   R_TEMP_L

    ldi  R_TEMP_H,      e_tx_config_2
    rcall EEPROM_WRITE_BYTE

    ret

;------------------------------------------------

 RFM12BP_CONF_DEV_1:

    lds  R_TEMP_L,      s_para_1
    andi R_TEMP_L,      0xF0

    in   R_TEMP_H,      TX_CONFIG_1
    andi R_TEMP_H,      0x0F

    or   R_TEMP_L,      R_TEMP_H
    out  TX_CONFIG_1,   R_TEMP_L

    ldi  R_TEMP_H,      e_tx_config_1
    rcall EEPROM_WRITE_BYTE

    ret

;------------------------------------------------

 RFM12BP_CONF_DEV_2:

    lds  R_TEMP_L,      s_para_1
    andi R_TEMP_L,      0xF0

    in   R_TEMP_H,      TX_CONFIG_2
    andi R_TEMP_H,      0x0F

    or   R_TEMP_L,      R_TEMP_H
    out  TX_CONFIG_2,   R_TEMP_L

    ldi  R_TEMP_H,      e_tx_config_2
    rcall EEPROM_WRITE_BYTE

    ret

;------------------------------------------------

 RFM12BP_CONF_BBW:

    lds  R_PARA_L,      s_para_1
    andi R_PARA_L,      0b11100000

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_READ_BYTE

    andi R_TEMP_L,      0b00011111
    or   R_TEMP_L,      R_PARA_L

    ldi  R_TEMP_H,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)

    M_SPI_TRANSFER

 .ifdef __RFM12BP_REQUEST__

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_WRITE_BYTE

 .endif

 .ifdef __RFM12BP_RESPONSE__

    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)
    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      e_cmd_rx_config
    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_WRITE_BYTE

    M_SET_TIMER_2 CONFIRM_TIMEOUT_6S

 .endif

    ret

;------------------------------------------------

 RFM12BP_CONF_LNAG:

    lds  R_PARA_L,      s_para_1
    andi R_PARA_L,      0b00011000

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_READ_BYTE

    andi R_TEMP_L,      0b11100111
    or   R_TEMP_L,      R_PARA_L

    ldi  R_TEMP_H,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)

    M_SPI_TRANSFER

 .ifdef __RFM12BP_REQUEST__

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_WRITE_BYTE

 .endif

 .ifdef __RFM12BP_RESPONSE__

    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)
    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      e_cmd_rx_config
    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_WRITE_BYTE

    M_SET_TIMER_2 CONFIRM_TIMEOUT_6S

 .endif

    ret

;------------------------------------------------

 RFM12BP_CONF_DRSSI:

    lds  R_PARA_L,      s_para_1
    andi R_PARA_L,      0b00000111

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_READ_BYTE

    andi R_TEMP_L,      0b11111000
    or   R_TEMP_L,      R_PARA_L

    ldi  R_TEMP_H,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)

    M_SPI_TRANSFER

 .ifdef __RFM12BP_REQUEST__

    ldi  R_TEMP_H,      e_cmd_rx_config
    rcall EEPROM_WRITE_BYTE

 .endif

 .ifdef __RFM12BP_RESPONSE__

    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      high(CMD_RX_CONFIG | PIN_16_VDI | VDI_SLOW)
    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      e_cmd_rx_config
    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_WRITE_BYTE

    M_SET_TIMER_2 CONFIRM_TIMEOUT_6S

 .endif

    ret

;------------------------------------------------

 RFM12BP_CONF_DQD:

    lds  R_PARA_L,      s_para_1
    andi R_PARA_L,      0b00000111

    ldi  R_TEMP_L,      low(CMD_SET_FILTER | CLK_REC_AUTO | DIG_FILTER)
    ldi  R_TEMP_H,      high(CMD_SET_FILTER | CLK_REC_AUTO | DIG_FILTER)

    or   R_TEMP_L,      R_PARA_L

    M_SPI_TRANSFER

 .ifdef __RFM12BP_REQUEST__

    ldi  R_TEMP_H,      e_cmd_set_filter
    rcall EEPROM_WRITE_BYTE

 .endif

 .ifdef __RFM12BP_RESPONSE__

    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      high(CMD_SET_FILTER | CLK_REC_AUTO | DIG_FILTER)
    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      e_cmd_set_filter
    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_WRITE_BYTE

    M_SET_TIMER_2 CONFIRM_TIMEOUT_6S

 .endif

    ret

;------------------------------------------------

 RFM12BP_CONFIRM:

 .ifdef __RFM12BP_RESPONSE__

    M_CHECK_TIMER_2
    rjmp NO_CONFIRM

    M_STOP_TIMER_2
    cbi  STATUS_REG,    NEW_INIT

    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_READ_BYTE
    mov  R_DATA_L,      R_TEMP_L

    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_READ_BYTE
    mov  R_TEMP_H,      R_DATA_L
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_READ_BYTE
    mov  R_TEMP_H,      R_DATA_L
    inc  R_TEMP_H
    rcall EEPROM_WRITE_BYTE

    clr  R_TEMP_L
    ldi  R_TEMP_H,      e_temp_value_l
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_H,      e_temp_value_h
    rcall EEPROM_WRITE_BYTE

    ldi  R_TEMP_L,      e_end
    ldi  R_TEMP_H,      e_temp_address
    rcall EEPROM_WRITE_BYTE

 NO_CONFIRM:

 .endif

    ret

;------------------------------------------------

 RFM12BP_DEFAULT_CONFIGURE_TABLE:
 .dw (CMD_SET_CONF   | CAPACITOR_12_5  | EN_TX_REG  | EN_RX_BUF | FREQ_BAND_868)
 .dw (CMD_SET_POWER  | EN_CRYSTAL      | DI_CLK_PIN | EN_SYNTH)
 .dw (CMD_SET_FREQ   | FREQ_869_525)
 .dw (CMD_SET_BR     | BIT_RATE_1_2)
 .dw (CMD_RX_CONFIG  | BAND_WIDTH_134  | PIN_16_VDI | DRSSI_103 | VDI_SLOW | LNA_GAIN_0)
 .dw (CMD_SET_FILTER | CLK_REC_AUTO    | DIG_FILTER | DQD_7)
 .dw (CMD_FIFO_RESET | FIFO_SYNC_WORD  | EN_HI_SENS | FIFO_INT_LEVEL_8)
 .dw (CMD_SYNC_PAT   | FRAME_START_L)
 .dw (CMD_AFC        | EN_AFC_ACCURACY | EN_AFC_REG | AFC_INDEPEND | EN_AFC_FUNC)
 .dw (CMD_TX_CONFIG  | OUTPUT_POWER_9  | FREQ_DEVIATION_30 )
 .dw (CMD_SET_PLL    | DI_DITHERING_PLL)
 .dw (CMD_WAKE_UP_TIME) ; disable
 .dw (CMD_LOW_DUTY)     ; disable
 .dw (CMD_BATT_CLK_DIV) ; disable
 RFM12BP_DEFAULT_CONFIGURE_TABLE_END:

 RFM12BP_LOAD_DEF:

    ldi  R_ZP_L,        low(RFM12BP_DEFAULT_CONFIGURE_TABLE << 1)
    ldi  R_ZP_H,        high(RFM12BP_DEFAULT_CONFIGURE_TABLE << 1)

    ldi  R_PARA_L,      low(RFM12BP_DEFAULT_CONFIGURE_TABLE_END << 1)
    ldi  R_PARA_H,      high(RFM12BP_DEFAULT_CONFIGURE_TABLE_END << 1)

 RFM12BP_LOAD_DEFAULT_CONFIG_LOOP:

    lpm  R_TEMP_L,      Z+
    lpm  R_TEMP_H,      Z+

    M_SPI_TRANSFER

;               op < op
    cp   R_ZP_L,        R_PARA_L
    cpc  R_ZP_H,        R_PARA_H
    brlo RFM12BP_LOAD_DEFAULT_CONFIG_LOOP

    ldi  R_TEMP_L,      (OUTPUT_POWER_6 | FREQ_DEVIATION_45)
    out  TX_CONFIG_1,   R_TEMP_L

    ldi  R_TEMP_L,      (OUTPUT_POWER_9 | FREQ_DEVIATION_30)
    out  TX_CONFIG_2,   R_TEMP_L

 ret

;------------------------------------------------

 RFM12BP_REMOTE_CMD:

    M_SPI_REMOTE_CMD

    ret

;------------------------------------------------

 RFM12BP_MODUL_RES:

    rcall RFM12BP_INIT

    ret

;------------------------------------------------

 RFM12BP_COMPL_RES:

    rjmp RESET

    ret

;------------------------------------------------

 TEST_LEDS:

    M_TEST_LEDS

    ret

;------------------------------------------------

 TEST_LED_DEBUG_1:

    M_LED_DEBUG_1_ON
    M_SET_TIMER_1 TIME_1S
    M_WAIT_TIMER_1
    M_LED_DEBUG_1_OFF

 ret

;------------------------------------------------

 TEST_LED_DEBUG_2:

    M_LED_DEBUG_2_ON
    M_SET_TIMER_1 TIME_1S
    M_WAIT_TIMER_1
    M_LED_DEBUG_2_OFF

    ret

;------------------------------------------------

 RFM12BP_END:

    ret

;------------------------------------------------

 .endmacro

;------------------------------------------------

 .endif

;-----------------------------------------------
