;*** receive.inc ***;
;------------------------------------------------
; Author: Torsten Knorr
; E-mail: create-soft@freenet.de
; copyright(C) 2016-2017 by Torsten Knorr
;------------------------------------------------

 .ifndef __RFM12BP_RECEIVE__
 .equ    __RFM12BP_RECEIVE__ = 1

 .include "RFM12BP.inc"

;------------------------------------------------
; RFM12BP PIN 11 DCLK/CFIL/FFIT -> ATmega48 PD3 INT1

 .macro M_INIT_INT1_RECEIVE

; Interrupt 1 sense control.
; ISC11 ISC10 Description
;     0     0 The low level of INT1 generates an interrupt request
;     0     1 Any logical change on INT1 generates an interrupt request
;     1     0 The falling edge of INT1 generates an interrupt request
;     1     1 The rising edge of INT1 generates an interrupt request
    lds  R_TEMP_L,      EICRA
    sbr  R_TEMP_L,      (1 << ISC11) | (1 << ISC10)
    sts  EICRA,         R_TEMP_L

    cbi  EIMSK,         INT1_EN_RX

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_INT1_RECEIVE

    in   RI_SREG,       SREG

    cbi  SPI_PORT,      SPI_SS

; send fifo read command
    ldi  RI_TX_BYTE,    high(CMD_FIFO_READ)
    out  SPDR,          RI_TX_BYTE
    M_SPI_WAIT_READY
; store received data
    in   RI_RX_BYTE,    SPDR

; read fifo
    ldi  RI_TX_BYTE,    0x00
    out  SPDR,          RI_TX_BYTE
    M_SPI_WAIT_READY
    in   RI_RX_BYTE,     SPDR

    sbi  SPI_PORT,      SPI_SS

; check pointer     >=
    cp   RI_XP_RTX_L,   RI_XP_END_L
    cpc  RI_XP_RTX_H,   RI_XP_END_H
    brsh RFM12BP_INT1_RECEIVE_RETURN
    st   X+,            RI_RX_BYTE

 RFM12BP_INT1_RECEIVE_RETURN:
    out  SREG,          RI_SREG
    reti

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_START_LISTEN_BAND_BUSY

    M_LED_LISTEN_ON
    M_SPI_MT_CMD (CMD_SET_POWER | EN_RX | EN_BB_CIRCUIT | EN_CRYSTAL | EN_SYNTH | DI_CLK_PIN)
    sbi  PORTC,         RFM12BP_EN_RX

    M_RFM12BP_START_VDI

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_LISTEN

    ldi  R_PARA_L,      low(@0)
    ldi  R_PARA_H,      high(@0)
    ldi  R_ZP_L,        low(@1)
    ldi  R_ZP_H,        high(@1)

    movw R_COUNT_L,     R_ZP_L
    add  R_COUNT_L,     R_PARA_L
    adc  R_COUNT_H,     R_PARA_H

; clear RX buffer
    clr  R_DATA_L
 RFM12BP_CLEAR_RX_BUFF_LOOP:

    st   Z+,            R_DATA_L

; check pointer     >=
    cp   R_ZP_L,        R_COUNT_L
    cpc  R_ZP_H,        R_COUNT_H
    brlo RFM12BP_CLEAR_RX_BUFF_LOOP

    M_SPI_MT_CMD (CMD_SET_POWER | EN_RX | EN_BB_CIRCUIT | EN_CRYSTAL | EN_SYNTH | DI_CLK_PIN)
    sbi  PORTC,         RFM12BP_EN_RX

    M_RFM12BP_START_VDI

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_RECEIVE

    M_SPI_MT_CMD (CMD_FIFO_RESET | FIFO_SYNC_WORD | FIFO_INT_LEVEL_8 | DI_HI_SENS)
    M_SPI_MT_CMD (CMD_FIFO_RESET | FIFO_SYNC_WORD | FIFO_INT_LEVEL_8 | DI_HI_SENS | EN_FIFO_FILL)

    ldi  R_PARA_L,      low(@0)
    ldi  R_PARA_H,      high(@0)
    ldi  RI_XP_RTX_L,   low(@1)
    ldi  RI_XP_RTX_H,   high(@1)

    movw RI_XP_END_L,   RI_XP_RTX_L
    add  RI_XP_END_L,   R_PARA_L
    adc  RI_XP_END_H,   R_PARA_H

; reset interrupt flag
    sbi  EIFR,          INTF1
; enable INT1 receive
    sbi  EIMSK,         INT1_EN_RX

    M_SET_RX_TIMEOUT (@0 + BYTES_PREAMBLE + BYTES_ID + BYTES_FINALE)

 RFM12BP_WAIT_RECIEVE:
    wdr

    M_CHECK_TIMER_1
    rjmp RFM12BP_END_RECEIVE

    M_RFM12BP_CHECK_VDI
    rjmp RFM12BP_END_RECEIVE

;                   <
    cp   RI_XP_RTX_L,   RI_XP_END_L
    cpc  RI_XP_RTX_H,   RI_XP_END_H
    brlo RFM12BP_WAIT_RECIEVE

; disable INT1 receive
    cbi  EIMSK,         INT1_EN_RX

    M_RFM12BP_WAIT_VDI_LOW

 RFM12BP_END_RECEIVE:

    M_RFM12BP_STOP_VDI
    M_STOP_TIMER_1

    cbi  PORTC,         RFM12BP_EN_RX
    M_SPI_MT_CMD (CMD_SET_POWER  | EN_SYNTH  | EN_CRYSTAL | DI_CLK_PIN)

 .endmacro

;------------------------------------------------

 .endif

;------------------------------------------------
